#!/usr/bin/env perl

# Password conversion utility for CGIForum.
#
# Starting with CGIForum 1.10, a more secure encryption method is used
# to store user passwords by default. An advantage of the method is
# that knowing the encrypted password does not enable an attacker to
# derive the original password. A disadvantage is that if users forget
# their passwords, there's no way to recover them.
#
# This script is only useful for users who want to upgrade existing
# CGIForum installations (< 1.10). If you're a new user, you needn't
# bother, as the more secure method is already enabled in the current
# cgiforum.conf.
#
#
# To switch your user passwords to the new scheme, first make a backup
# of your "cgiforum/users" directory, which is used to store user
# data. Then, copy this script, "pwlconvert.pl", to "cgiforum/users",
# and invoke it via
#
#     perl pwlconvert.pl
#
# Follow the instructions. After the conversion is completed, add the
# setting
#
# encryption_method=crypt
#
# to your cgiforum.conf
#
# If you want to have/keep the previous scheme (enabling easy password
# recovery) with new versions of CGIForum, either set
#
# encryption_method=xor12
#
# or say nothing about encryption_method in cgiforum.conf
#



use strict;
use warnings;

my @pwls = <*.pwl>;

scalar @pwls or die "no *.pwl files in current directory\n";

print "\nNumber of *.pwl files: " . scalar @pwls . ". Please make a backup!\nPress RETURN to convert or Ctrl + c to quit.\n";

my $inp = <STDIN>;

sub encrypt_xor12
{
    my @args = split //, shift;
    my $retval = '';
    foreach my $c (@args) { $retval .= chr(ord($c) ^ 0x12); }
    return $retval;
}


sub encrypt_crypt
{
    my $salt = join '', ('.', '/', 0..9, 'A'..'Z', 'a'..'z')[rand 64,rand 64];
    return crypt shift, $salt;
}



foreach my $f (<*.pwl>) {
    open PWL, $f or die "couldn't read from $f\n";
    print "converting: $f\n";
    my $pw = <PWL>;
    close PWL;
    $pw = encrypt_xor12 $pw;
    #print "decrypted: $pw\n";

    $pw = encrypt_crypt $pw;
    #print "encrypted: $pw\n\n";
    open PWL, ">$f" or die "couldn't write to $f\n";
    print PWL "$pw" and close PWL;
}
